/*
 *      configdialog.h
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef CONFIGDIALOG_H
#define CONFIGDIALOG_H

#include <QDialog>
#include <QBoxLayout>
#include <QTabWidget>
#include <QCheckBox>
#include <QSpinBox>
#include <QLineEdit>
#include <QPushButton>
#include <QListWidget>
#include <QLabel>
#include <QComboBox>
#include <QTextEdit>
#include <QTreeWidget>
#include "preferences.h"
#include "tabitems/tabwidget.h"
#include "widgets/colorbutton.h"

class ConfigDialog : public QDialog
{
    Q_OBJECT
public:
    ConfigDialog(QWidget *parent = 0);
signals:
    void restartApp();
private slots:
    void accept();
    void slotLogging();
    void slotLogPath();
    void slotThemeRoles(QListWidgetItem *current);
    void slotAddRoleTheme();
    void slotDeleteRoleTheme();
    void slotTabColor();
    void slotTabPosition();
    void slotIrcTextColor();
    void slotSound();
    void slotSoundConnect();
    void slotSoundDisconnect();
    void slotSoundMessage();
    void slotPath();
    void slotPlay();
    void slotNotify();
    void slotShowNotify();
    void slotUseSmileys();
    void slotAddSmiley();
    void slotModifySmiley();
    void slotDeleteSmiley();
    void slotImportSmileys();
    void slotExportSmileys();
    void slotModifyIrcFont();
    void slotReconnect();
    void slotAutoload();
    void slotAutoloadPath();
    void slotLoadLog();
    void slotAddFriend();
    void slotModifyFriend();
    void slotDeleteFriend();
    void slotAddIgnored();
    void slotModifyIgnored();
    void slotDeleteIgnored();
    void slotAddCommand();
    void slotDeleteCommand();
    void slotDccPath();
    void slotModifyKeybindings();
    void slotRestoreDefaultKeybindings();
private:
    bool m_showImportwarning;
    //GeneralTab widgets
    QCheckBox *m_oneInstance, *m_logging, *m_disableSort, *m_reconnect;
    QCheckBox *m_closeToTray, *m_useSpell, *m_ownServerWindow, *m_autoload;
    QLabel *m_logPath, *m_autoloadPath;
    QPushButton *m_setLogPath, *m_setAutoloadPath;
    QSpinBox *m_numberAttempt, *m_delayAttempt, *m_maxAway;
    //IconsTab widgets
    QListWidget *m_themesRoleList;
    QLabel *m_iconAway, *m_iconAdmin, *m_iconOwner, *m_iconHalfop, *m_iconNormal, *m_iconOp, *m_iconVoice;
    QPushButton *m_themeRoleAdd, *m_themeRoleDelete;
    QCheckBox *m_sortUserMode;
    //LookTab widgets
    ColorButton *m_unreadtabColor, *m_hltabColor;
    QComboBox *m_tabPosition;
    TabWidget *m_lookTabs;
    //IrcTextTab widgets
    ColorButton *m_textColor, *m_backroundColor, *m_userColor, *m_actionColor, *m_errorColor;
    ColorButton *m_hilightColor, *m_linkColor, *m_mymsgColor, *m_logColor, *m_noticeColor;
    QTextEdit *m_ircText;
    QLabel *m_ircFont;
    QPushButton *m_ircFontModify;
    QLineEdit *m_nickCompletionChar;
    QCheckBox *m_sameCmd, *m_sameList, *m_coloredNick;
    QCheckBox *m_stripColors, *m_loadLog;
    QSpinBox *m_loadLines;
    //NotificationTab widgets
    QList<UserInfo> m_friends;
    QTreeWidget *m_friendsList;
    QPushButton *m_friendAdd, *m_friendModify, *m_friendDelete;
    QCheckBox *m_sounds, *m_soundConnect, *m_soundDisconnect, *m_soundMessage;
    QLabel *m_pathConnect, *m_pathDisconnect, *m_pathMessage;
    QPushButton *m_setpathConnect, *m_setpathDisconnect, *m_setpathMessage;
    QPushButton *m_playConnect, *m_playDisconnect, *m_playMessage;
    QCheckBox *m_notify, *m_notifyConnect, *m_notifyDisconnect, *m_notifyMessage, *m_notifyTray;
    QComboBox *m_notifyPosition;
    QPushButton *m_showNotify;
    //SmileysTab widgets
    QCheckBox *m_useSmileys;
    QMap<QString,QString> m_smileysMap;
    QListWidget *m_smileysList;
    QPushButton *m_addSmiley, *m_modifySmiley, *m_deleteSmiley, *m_importSmiley, *m_exportSmiley;
    //Ignore widgets
    QList<UserInfo> m_ignored;
    QTreeWidget *m_ignoredList;
    QPushButton *m_ignoredAdd, *m_ignoredModify, *m_ignoredDelete;
    QStringList m_commands;
    QListWidget *m_commandsList;
    QPushButton *m_commandAdd, *m_commandDelete;
    //DCC widgets
    QLabel *m_dccPath;
    QPushButton *m_setDccPath;
    QLineEdit *m_dccIP;
    QSpinBox *m_dccPortD, *m_dccPortH, *m_dccTimeout;
    QCheckBox *m_autoDccChat, *m_autoDccFile;

    void createGeneralTab(QTabWidget *tabbook);
    void createNicklistTab(QTabWidget *tabbook);
    void createLookTab(QTabWidget *tabbook);
    void createIrctextTab(QTabWidget *tabbook);
    void createNotificationTab(QTabWidget *tabbook);
    void createSmileysTab(QTabWidget *tabbook);
    void createIgnoreTab(QTabWidget *tabbook);
    void createDccTab(QTabWidget *tabbook);
    void createKeybindingsTab(QTabWidget *tabbook);
    void updateTabPosition();
    void updateIrcText();
    void fillSmileys();
    bool nickExists(const QString &nick, bool frien);
    int nickIndex(const QString &nick, bool frien);
};

class SmileyDialog : public QDialog
{
    Q_OBJECT
public:
    SmileyDialog(const QString &title, const QString &smiley, const QString &path, QWidget *parent = 0);

    QString getSmiley() { return m_smiley->text(); }
    QString getPath() { return m_path->text(); }
private slots:
    void slotPath();
private:
    QLabel *m_path, *m_preview;
    QPushButton *m_selectPath;
    QLineEdit *m_smiley;
};

#endif // CONFIGDIALOG_H
