/*
 *      dccsenddialog.cpp
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QtGui>
#if(QT_VERSION > 0x050000)
#include <QtWidgets>
#endif
#include "dccsenddialog.h"
#include "config.h"
#include "i18n.h"

DccSendDialog::DccSendDialog(const QString &nick, QWidget *parent) :
    QDialog(parent)
{
    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Cancel);
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    QPushButton *buttonSend = new QPushButton(_("&Send file"));
    buttons->addButton(buttonSend, QDialogButtonBox::AcceptRole);
    QVBoxLayout *mainLayout = new QVBoxLayout;
    QGridLayout *matrix = new QGridLayout;
    matrix->addWidget(new QLabel(_("File:")), 0, 0);
    m_file = new QLineEdit;
    matrix->addWidget(m_file, 0, 1);
    QPushButton *buttonFile = new QPushButton("...");
    connect(buttonFile, SIGNAL(clicked()), this, SLOT(selectFile()));
    matrix->addWidget(buttonFile, 0, 2);
    matrix->addWidget(new QLabel(_("Send passive")), 1, 0);
    m_passive = new QCheckBox;
    matrix->addWidget(m_passive, 1, 1);
    mainLayout->addLayout(matrix);
    mainLayout->addWidget(buttons);
    setLayout(mainLayout);
    setWindowTitle(_("Send file to %1").arg(nick));
}

void DccSendDialog::accept()
{
    QDialog::accept();
}

void DccSendDialog::selectFile()
{
    QFileDialog::Options options = QFileDialog::DontResolveSymlinks;
    QString path = QFileDialog::getOpenFileName(this,
                                _("Select file"),
#if(QT_VERSION > 0x050000)
                                QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0),
#else
                                QDesktopServices::storageLocation(QDesktopServices::HomeLocation),
#endif
                                "",
                                NULL,
                                options);
    if(!path.isEmpty())
    {
        m_file->setText(path);
    }
}

bool DccSendDialog::passive()
{
    return m_passive->isChecked();
}

QString DccSendDialog::file() const
{
    return m_file->text();
}
