/*
 *      listdialog.cpp
 *
 *      Copyright 2015 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "listdialog.h"
#include "dxutils.h"
#include "config.h"
#include "i18n.h"

ListDialog::ListDialog(IrcEngine *engine, QStringList channels, QWidget *parent)
: QDialog(parent), m_engine(engine), m_channels(channels)
{
    QVBoxLayout *mainFrame = new QVBoxLayout(this);

    QGridLayout *matrix = new QGridLayout;
    matrix->addWidget(new QLabel(_("Search")), 0, 0);
    m_search = new QLineEdit;
    connect(m_search, SIGNAL(textChanged(QString)), this, SLOT(slotSearch()));
    matrix->addWidget(m_search, 0, 1);
    mainFrame->addLayout(matrix);

    QStringList headers;
    headers << _("Channel") << _("Members") << _("Topic");
    m_table = new QTreeWidget;
    m_table->setHeaderLabels(headers);
    m_table->setSelectionBehavior(QAbstractItemView::SelectRows);
    m_table->setAlternatingRowColors(true);
    m_table->setRootIsDecorated(false);
    m_table->setSelectionMode(QAbstractItemView::SingleSelection);
    m_table->setSortingEnabled(true);
    connect(m_table, SIGNAL(itemDoubleClicked(QTreeWidgetItem*,int)), this, SLOT(slotJoin()));
    QHeaderView *header = m_table->header();
    header->resizeSection(0, 200);
    header->resizeSection(1, 75);
    header->resizeSection(2, 400);
    mainFrame->addWidget(m_table);

    QDialogButtonBox *buttons = new QDialogButtonBox();
    m_buttonJoin = new QPushButton(_("&Join"));
    m_buttonJoin->setEnabled(channels.size()?true:false);
    connect(m_buttonJoin, SIGNAL(clicked()), this, SLOT(slotJoin()));
    buttons->addButton(m_buttonJoin, QDialogButtonBox::ActionRole);
    m_buttonClose = new QPushButton(_("&Close"));
    connect(m_buttonClose, SIGNAL(clicked()), this, SLOT(slotClose()));
    buttons->addButton(m_buttonClose, QDialogButtonBox::ActionRole);
    mainFrame->addWidget(buttons);
    
    setLayout(mainFrame);
    setWindowTitle(_("Channels list")+" "+m_engine->getNetworkName());
    setMinimumWidth(700);

    fillChannelList();
    m_engine->clearChannelsList();
    m_table->sortItems(0, Qt::AscendingOrder);
}

void ListDialog::slotJoin()
{
    m_engine->sendJoin(m_table->currentItem()->text(0));
    accept();
}

void ListDialog::slotClose()
{
    accept();
}

void ListDialog::slotSearch()
{
    fillChannelList();
}

void ListDialog::fillChannelList()
{
    m_table->clear();
    QTreeWidgetItem *current = NULL;
    if(m_search->text().isEmpty())
    {
        for(int i=0; i < m_channels.size(); ++i)
        {
            QTreeWidgetItem *item = new QTreeWidgetItem(m_table);
            item->setText(0, dxutils::getParam(m_channels[i], 1, false));
            item->setData(1, Qt::EditRole, dxutils::getParam(m_channels[i], 2, false).toInt());
            item->setText(2, dxutils::stripColors(dxutils::getParam(m_channels[i], 3, true).section(':',1), true));
            if(i == 0) current = item;
        }
    }
    else
    {
        for(int i=0; i < m_channels.size(); ++i)
        {
            if(dxutils::getParam(m_channels[i], 1, false).contains(m_search->text()))
            {
                QTreeWidgetItem *item = new QTreeWidgetItem(m_table);
                item->setText(0, dxutils::getParam(m_channels[i], 1, false));
                item->setData(1, Qt::EditRole, dxutils::getParam(m_channels[i], 2, false).toInt());
                item->setText(2, dxutils::stripColors(dxutils::getParam(m_channels[i], 3, true).section(':',1), true));
                if(!current) current = item;
            }
        }
    }
    if(current) m_table->setCurrentItem(current);
    m_buttonJoin->setEnabled(current);
}
