/*
 *      dxirc.h
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef DXIRC_H
#define DXIRC_H

#include <QtGui>
#if(QT_VERSION > 0x050000)
#include <QtWidgets>
#endif
#include "tabitems/tabwidget.h"
#include "tabitems/fttabitem.h"
#include "logviewer/logdialog.h"
#include "engine/ircengine.h"
#include "engine/dccengine.h"

class dxirc : public QMainWindow
{
    Q_OBJECT
public:
    explicit dxirc(QWidget *parent = 0);
    ~dxirc();

    static dxirc* instance();
    QList<LuaScript> & getScripts() { return m_scripts; }
    QList<DccFile> & getDccFiles() { return m_dccfilesList; }
    int loadLuaScript(QString, bool=true);
    int unloadLuaScript(QString);
    static int onLuaAddCommand(lua_State*);
    static int onLuaAddEvent(lua_State*);
    static int onLuaRemoveName(lua_State*);
    static int onLuaCommand(lua_State*);
    static int onLuaPrint(lua_State*);
    static int onLuaGetServers(lua_State*);
    static int onLuaGetTab(lua_State*);
    static int onLuaGetCurrentTab(lua_State*);
    static int onLuaGetVersion(lua_State*);
    static int onLuaGetTabInfo(lua_State*);
    static int onLuaSetTab(lua_State*);
    static int onLuaCreateTab(lua_State*);
    static int onLuaGetTabCount(lua_State*);
    static int onLuaClear(lua_State*);
    static int onLuaGetChannelNames(lua_State*);
private:
    static dxirc *_pThis;
    QSystemTrayIcon *m_trayIcon;
    TabWidget *m_tabbook;
    QStatusBar *m_statusBar;
    QList<LuaScript> m_scripts;
    QList<IrcEngine*> m_ircengines;
    QList<DccEngine*> m_dccengines;
    QList<LuaScriptEvent> m_scriptEvents;
    QList<DccFile> m_dccfilesList;
    FtTabItem *m_ft;
    quint32 m_lastID;
    int m_lastToken;
    QAction *m_disconnect;
    QAction *m_reconnect;
    QAction *m_closeTab;
    QAction *m_clearTab;
    QAction *m_clearTabs;
    QAction *m_logviewer;
    QAction *m_users;
    QAction *m_status;
#ifdef HAVE_ENCHANT
    QAction *m_spellCombo;
#endif
    QTimer *m_statusTimer;
    LogDialog *m_logs;
    QTimer *m_awayTimer;
    dxSocket *m_versionSocket;

    void createActions();
    void createMenus();
    void createTrayIcon();
    void updateTabPosition();
    void updateMenus();
    void updateServerWindow();
    void updateStatus(QString text);
    void sortTabs();
    quint32 createIrcTab(const QString& tabtext, TYPE typ, IrcEngine* engine, QIcon icon);
    quint32 createDccTab(const QString &mynick, const QString &nick, const QString &address, int portD, int portH=0, bool listen=false, IrcEngine *engine=NULL);
    quint32 createBoatsTab(const QString& tabtext, IrcEngine* engine, bool first);
    void connectServer(QString hostname, int port, QString pass, QString nick, QString rname, QString channels, QString commands, bool ssl);
    bool serverExist(QString server, int port, QString nick);
    int getServerTab(IrcEngine *server);
    bool isLastTab(IrcEngine *server);
    bool tabExist(IrcEngine *server, QString name);
    bool tabExist(QString name);
    int getCurrentTabId();
    void setCurrentTabById(int id);
    int getTabId(IrcEngine *server, QString name);
    int getTabId(QString name);
    int getTabIndex(IrcEngine *server, QString name);
    QWidget* getTabItemById(int id);
    bool isValidTabId(int id);
    bool isIddxTabItem(int id);
    void appendIrcText(QString text, bool logLine=true);
    void appendIrcStyledText(QString text, int style, bool logLine=true);
    bool hasTetrisTab();
    bool hasLuaAll(const QString &);
    bool hasAllCommand();
    bool hasOwnMsg();
    void autoloadScripts();
    void sendNewTab(IrcEngine *server, const QString &name, int id, bool isTetris, bool isDccTab, TYPE type);
    bool isFriend(QString nick, QString on, QString network);
    QString getUniqueName(const QString &path, const QString &name, const QString &extension);
    bool isForResume(const QString& name);
    void setTrayIcon(bool newmsg=false, bool newfile=false);
private slots:
    void slotCmdAbout();
    void slotCmdQuit();
    void slotIconActivated(QSystemTrayIcon::ActivationReason);
    void slotCmdServers();
    void slotCmdConnect();
    void slotCmdReconnect();
    void slotCmdDisconnect();
    void slotCmdScripts();
    void slotCmdLogs();
    void slotCmdCloseTab();
    void slotCmdClearTab();
    void slotCmdClearTabs();
    void slotCmdUsersList();
    void slotCmdSpell();
    void slotCmdStatus();
    void slotCmdAlias();
    void slotCmdPreferences();
    void slotCmdTransfers();
    void slotCmdCheckVersion();
    void slotCmdHelp();
    void slotTabChanged(int index);
    void slotCloseFTTab();
    void slotCancelFT(int index);
    void slotStatusTime();
    void slotAwayTime();
    void slotUnreadTab();
    void slotNextTab();
    void slotVersionConnected();
    void slotVersionCanRead();
    void slotVersionError(QString);
    void slotForceFocus();
    void slotMoveTab();
    void slotSelectTab();
    void slotRestartApp();
    void slotCmdList();
    //IrcEngine slots
    void slotIrcNewChannel(QString channel);
    void slotIrcQuery(QString with);
    void slotIrcDccChat(QString with, QString address, int port);
    void slotIrcInvite(QString nick, QString channel);
    void slotIrcEndMotd();
    void slotIrcPrivmsgAndAction(QString nick, QString to, QString msg);
    void slotIrcPart(QString nick, QString channel);
    void slotIrcKick(QString nick, QString to, QString channel);
    void slotIrcDisconnect(QString msg);
    void slotIrcConnectAndReconnect(QString msg);
    void slotIrcJoin(QString nick, QString channel);
    void slotIrcQuit(QString nick);
    void slotIrcDccIn(QString nick, QString ip, QString port, QString filename, QString size);
    void slotIrcDccMyToken(QString ip, int port, int token);
    void slotIrcDccToken(QString nick, QString filename, QString size, int token);
    void slotIrcDccResume(QString nick, QString filename, QString port, QString position);
    void slotIrcDccPresume(int token, QString position);
    void slotIrcDccAccept(QString nick, QString filename, QString port, QString position);
    void slotIrcDccPaccept(int token, QString position);
    void slotIrcRaw(QString line);
    void slotIrcBoatsInvite(QString nick);
    void slotIrcBoatsAccept(QString nick);
    void slotIrc323();
    //IrcTabItem and DccTabItem slots
    void slotHighlight(TYPE type);
    void slotNewmsg(TYPE type);
    void slotLuaRequest(LuaRequest lua);
    void slotNewtetris();
    void slotConnectServer(ServerInfo info);
    void slotIrcDccServer(IrcEngine *server, QString with);
    void slotIrcDccOut(IrcEngine *server, QString nick, QString file);
    void slotIrcDccPout(IrcEngine *server, QString nick, QString file);
    void slotIrcPrivmsgAndAction(IrcEngine*, QString nick, QString to, QString msg);
    void slotIrcQuery(IrcEngine *server, QString with);
    void slotOwnMsg(QString msg);
    void slotCommand(QString command);
    void slotAddIgnoreCommand(QString command);
    void slotRemoveIgnoreCommand(QString command);
    void slotAddIgnoreUser(QString text);
    void slotRemoveIgnoreUser(QString text);
    void slotNewlog(QString logpath);
    void slotNewname(QString name);
    //DccEngine
    void slotDccError(QString error);
    void slotDccPosition(DccFile dcc);
protected:
    void closeEvent(QCloseEvent *ev);
};

#endif // DXIRC_H
