/*
 *      dxutils.h
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef DXUTILS_H
#define DXUTILS_H

#include <QKeyEvent>
#include <QString>
#include <QMap>
#include "defs.h"
#include "config.h"
#ifdef HAVE_ENCHANT
namespace enchant
{
    class Dict;
}
typedef std::map<QString, enchant::Dict*> Checkers;
#endif

#define dxUtils dxutils::instance()

class dxutils
{
public:
    dxutils();
    virtual ~dxutils();

    static dxutils &instance();
    static QString checkThemePath(const QString &path);
    static QString checkThemesList(const QString &list);
    static QString getFileSize(quint64 size);
    static QString getFileSize(const QString &ssize);
    static QString fromStdString(const std::string &str);
    static std::string toStdString(const QString &str);
    static QString getNickColor(const QString &nick);
    static QString getSpeed(quint64 speed);
    static QString getRemaining(quint64 size, quint64 speed);
    void showNotify(QString text, int position=-1);
    void removeNotify();
    static void debugLine(const QString &line);
    static bool isRightLogFile(const QString &path, const QString &name);
    static QString getMonth(int month);
    static QString getMonthNumber(const QString &month);
    static void playFile(const QString &file);
    static QString createSmileys(QString text);
    static QString encrypt(const QString &text);
    static QString decrypt(const QString &text);
    static QString getParam(QString, int, bool);
    static QString getParam(QString, int, bool, const char&);
    static QString removeSpaces(const QString &text);
    static QString removeNonalphanumeric(const QString &text);
    static QString createModes(char sign, char mode, QString nicks);
    static bool isUtf8(const char *buf);
    bool haveNotify();
    void loadAlias();
    void setAliases(QMap<QString,QString>);
    QString getAlias(QString key);
    QMap<QString,QString> getAliases();
    int commandsNo();
    QString commandsAt(int);
    bool isCommand(const QString&);
    int dcccommandsNo();
    QString dcccommandsAt(int);
    bool isDccCommand(const QString&);
    void addScriptCommand(LuaScriptCommand command);
    bool removeScriptCommand(const QString &command);
    bool removeScriptCommands(const QString &script);
    bool isScriptCommand(const QString &command);
    QString availableCommands();
    QString availableDccCommands();
    QString availableScriptsCommands();
    QString availableScriptCommands(const QString &script);
    QString getHelpText(const QString &command);
    QString getFuncname(const QString &command);
    QString getScriptName(const QString &command);
    QString getStringIniEntry(const QString &section, const QString &key, const QString &def="");
    int getIntIniEntry(const QString &section, const QString &key, int def=0);
    bool getBoolIniEntry(const QString &section, const QString &key, bool def=false);
    QColor getColorIniEntry(const QString &section, const QString &key, QColor def=QColor(255,255,255));
    bool checkWord(QString word, QString lang);
    void loadAvailableSpellLangs();
    int getAvailableSpellLangsNum();
    QStringList getAvailableSpellLangs();
    QString getChannelLang(const QString &channel);
    static QString stamp2string(QString datestr, QString timestr);
    static QString normalizeLogLine(QString datestr, std::string line);
    static QString stripColors(const QString &text, const bool stripOther);
    static QColor getIrcColor(int code);
    static QString stripLogMark(const QString &text);
    static QColor getColorForStyle(int style);
    static QString createLinks(const QString &txt, bool removeThan=true);
    static QString parseText(const QString &txt, int stylenum, bool removeThan=true);
    static QString parseLogline(const QString &line, bool logstyle=false);
    static bool isBadchar(char c);
    static bool isDelimiter(char c);
    static QColor makeAwayColor(QColor clr);
    static QString modeToIcon(UserMode mode);
    static QString getUsername();
    static QString getIconForType(TYPE type);
    //dirName Path of directory to remove.
    static bool removeDir(const QString &dirName);
    static QString getLocalePath();
    static QString getDataPath();
    static QKeySequence getKeybinding(QKeyEvent *event);
    static QString wrapText(QString text, int width, char wrap=' ');
private:
    int m_notifiesHeight, m_notifyPosition;
    bool m_firstNotify, m_haveNotify;
    QString m_locale;
    QMap<QString,QString> m_aliases;
    QStringList m_commands;
    QStringList m_dcccommands;
    QStringList m_availableSpellLangs;
    QList<LuaScriptCommand> m_scriptCommands;
#ifdef HAVE_ENCHANT
    Checkers m_checkers;
#endif

    void fillCommands();
    QString getLocale();
    QString getDefaultLang();
};

#endif // DXUTILS_H
