/*
 *      notifywindow.cpp
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *      Copyright (c) 2008 by Rustam Chakin
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QApplication>
#include <QTimer>
#include <QDesktopWidget>
#include <QLabel>
#include <QHBoxLayout>
#include "notifywindow.h"
#include "preferences.h"
#include "dxutils.h"

NotifyWindow::NotifyWindow(const QString &body, int previous, int delay, QWidget *parent) :
    QWidget(parent), m_delay(delay), m_previous(previous)
{
    QLabel *icon = new QLabel;
    icon->setPixmap(QPixmap(":/icons/dxirc.png"));
    QLabel *text = new QLabel;
    text->setText(QString("<b>dxirc</b><br>%1").arg(body));
    QHBoxLayout *layout = new QHBoxLayout(this);
    layout->addWidget(icon);
    layout->addWidget(text);
    layout->addSpacerItem(new QSpacerItem(5, 5, QSizePolicy::Expanding, QSizePolicy::Minimum));
    setLayout(layout);
    setAttribute(Qt::WA_QuitOnClose, false);
    setAttribute(Qt::WA_DeleteOnClose, true);
    QTimer::singleShot(m_delay * 1000, this, SLOT(close()));
#if defined(Q_OS_DARWIN)
    setWindowFlags(windowFlags() | Qt::FramelessWindowHint);
#else
    setWindowFlags(windowFlags() | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint | Qt::ToolTip);
#endif
}

NotifyWindow::~NotifyWindow()
{
    dxUtils.removeNotify();
}

void NotifyWindow::showNotify(int position)
{
    show();
    QRect geometry = QApplication::desktop()->availableGeometry();
    int moveToPointX = 0;
    int moveToPointY = 0;
    if(position<0) position = preferences.m_notifyPosition;

    switch(position)
    {
    case 0:
        moveToPointX = geometry.left();
        moveToPointY = geometry.top() + m_previous;
        break;
    case 1:
        moveToPointX = geometry.right() - width();
        moveToPointY = geometry.top() + m_previous;
        break;
    case 2:
        moveToPointX = geometry.left();
        moveToPointY = geometry.bottom() - m_previous - height();
        break;
    default:
        moveToPointX = geometry.right() - width();
        moveToPointY = geometry.bottom() - m_previous - height();
    }
    move(moveToPointX, moveToPointY);
}

