/*
 *      linespell.h
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef LINESPELL_H
#define LINESPELL_H

#include <QTextEdit>
#include <QSyntaxHighlighter>
#include <QHash>
#include <QTextCharFormat>

class SpellHighlighter : public QSyntaxHighlighter
{
    Q_OBJECT
public:
    SpellHighlighter(QTextDocument *parent = 0);

    QString language() const;
    void setLanguage(const QString &lang);
protected:
    void highlightBlock(const QString &text);
private:
    QString m_spellLang;
};

class LineSpell : public QTextEdit
{
    Q_OBJECT
public:
    explicit LineSpell(QWidget *parent = 0);

    QSize minimumSizeHint() const;
    QSize sizeHint() const;
    void setUseSpell(bool spell);
    QString language() const;
    void setLanguage(const QString &lang);
    QString text() const;
    void setText(const QString &);
    int cursorPosition() const;
    void setCursorPosition(int);
private:
    bool m_useSpell;
    SpellHighlighter *m_spell;
protected:
    void keyPressEvent(QKeyEvent *e);
signals:
    void returnPressed();
};

#endif // LINESPELL_H
